@@ -5,7 +5,7 @@
 Manage GitHub repositories with ``repo-helper``.
 """
 #
-#  Copyright © 2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
+#  Copyright © 2020-2021 Dominic Davis-Foster <dominic@davis-foster.co.uk>
 #
 #  Permission is hereby granted, free of charge, to any person obtaining a copy
 #  of this software and associated documentation files (the "Software"), to deal
@@ -28,44 +28,40 @@ Manage GitHub repositories with ``repo-helper``.
 
 # stdlib
 import tempfile
-from base64 import b64encode
-from contextlib import contextmanager
 from getpass import getpass
 from typing import Iterator, Optional, Union
 
 # 3rd party
 import click
-from apeye import RequestsURL
 from consolekit.input import confirm
-from consolekit.terminal_colours import Fore, resolve_color_default
+from consolekit.terminal_colours import ColourTrilean, Fore, resolve_color_default
 from consolekit.utils import abort
+from deprecation_alias import deprecated
 from domdf_python_tools.paths import PathPlus
-from domdf_python_tools.stringlist import DelimitedList
 from domdf_python_tools.typing import PathLike
 from dulwich.errors import NotGitRepository
 from dulwich.porcelain import fetch
-from github import Github, GithubException
-from github.AuthenticatedUser import AuthenticatedUser
-from github.ContentFile import ContentFile
-from github.GithubException import UnknownObjectException
-from github.Organization import Organization
-from github.Repository import Repository
-from nacl import encoding, public  # type: ignore
+from github3 import GitHub, orgs, repos, users  # type: ignore
+from github3.exceptions import NotFoundError  # type: ignore
+from github3.repos import contents  # type: ignore
+from github3.repos.branch import Branch  # type: ignore
+from github3_utils import echo_rate_limit as _utils_echo_rate_limit
+from github3_utils import get_user, protect_branch, secrets
+from github3_utils.check_labels import check_status_labels
 from repo_helper.core import RepoHelper
 from repo_helper.files.ci_cd import ActionsManager, platform_ci_names
 from repo_helper.utils import set_gh_actions_versions
 from southwark.repo import Repo
-from typing_extensions import NoReturn
 
 # this package
-from repo_helper_github._types import _EditKwargs, _ExcData
+from repo_helper_github._github import Github
+from repo_helper_github._types import _EditKwargs
 from repo_helper_github.cli import github_command
-from repo_helper_github.options import token_option, version_callback
 
 __author__: str = "Dominic Davis-Foster"
-__copyright__: str = "2020 Dominic Davis-Foster"
+__copyright__: str = "2020-2021 Dominic Davis-Foster"
 __license__: str = "MIT License"
-__version__: str = "0.4.1"
+__version__: str = "0.5.0"
 __email__: str = "dominic@davis-foster.co.uk"
 
 __all__ = [
@@ -77,35 +73,18 @@ __all__ = [
 		"compile_required_checks",
 		]
 
-
-@contextmanager
-def echo_rate_limit(github: Github, verbose: bool = True):
-	"""
-	Contextmanager to echo the GitHub API rate limit before and after making a series of requests.
-
-	:param github:
-	:param verbose: If :py:obj:`False` no output will be printed.
-	"""
-
-	rate = github.get_rate_limit()
-	remaining_requests = rate.core.remaining
-
-	if not remaining_requests:
-		raise abort(f"No requests available! Resets at {rate.core.reset}")
-
-	if verbose:
-		click.echo(f"{remaining_requests} requests available.")
-
-	yield github
-
-	if verbose:
-		rate = github.get_rate_limit()
-		used_requests = remaining_requests - rate.core.remaining
-		click.echo(f"Used {used_requests} requests. {rate.core.remaining} remaining. Resets at {rate.core.reset}")
+echo_rate_limit = deprecated(
+		deprecated_in="0.5.0",
+		removed_in="1.0.0",
+		current_version=__version__,
+		details="Use the new 'github3-utils' package instead."
+		)(
+				_utils_echo_rate_limit
+				)
 
 
 def _lower(string: str) -> str:
-	return string.lower()
+	return string.lower().replace('_', '-')
 
 
 class GitHubManager(RepoHelper):
@@ -119,26 +98,24 @@ class GitHubManager(RepoHelper):
 	:param verbose: Whether to show information on the GitHub API rate limit.
 	:param colour: Whether to use coloured output.
 
-	.. versionchanged:: 0.4.1
-
-		Added the ``verbose`` and ``colour`` options.
+	.. versionchanged:: 0.5.0  Added the ``verbose`` and ``colour`` options.
 	"""  # noqa: D400
 
 	#:
-	github: Github
+	github: GitHub
 
 	verbose: bool
 	"""
 	Whether to show information on the GitHub API rate limit.
 
-	.. versionadded: 0.4.1
+	.. versionadded: 0.5.0
 	"""
 
-	colour: Optional[bool]
+	colour: ColourTrilean
 	"""
 	Whether to use coloured output.
 
-	.. versionadded: 0.4.1
+	.. versionadded: 0.5.0
 	"""
 
 	def __init__(
@@ -148,11 +125,11 @@ class GitHubManager(RepoHelper):
 			managed_message="This file is managed by 'repo_helper'. Don't edit it directly.",
 			*,
 			verbose: bool = False,
-			colour: Optional[bool] = True,
+			colour: ColourTrilean = True,
 			):
 		super().__init__(target_repo, managed_message)
 
-		self.github = Github(token)
+		self.github = Github(token=token)
 		self.verbose = verbose
 		self.colour = resolve_color_default(colour)
 
@@ -170,20 +147,26 @@ class GitHubManager(RepoHelper):
 		:param org: Whether the repository should be created for the organisation set as ``username``,
 			or for the authenticated user (default).
 
-		.. versionchanged:: 0.4.1
+		.. versionchanged:: 0.5.0
 
 			Removed the ``verbose`` option. Provide it to the class constructor instead.
 
-		.. versionchanged:: 0.4.1  Added the ``org`` argument.
+		.. versionchanged:: 0.5.0  Added the ``org`` argument.
 		"""
 
 		with self.echo_rate_limit():
 			user = self.get_org_or_user(org)
+			repo_name = self.templates.globals["repo_name"]
 
-			try:
-				repo = user.create_repo(self.templates.globals["repo_name"], **self.get_repo_kwargs())
-			except GithubException as e:
-				self.handle_exception(e)
+			repo: Optional[repos.Repository]
+
+			if org:
+				repo = user.create_repository(repo_name, **self.get_repo_kwargs())
+			else:
+				repo = self.github.create_repository(repo_name, **self.get_repo_kwargs())
+
+			if repo is None:
+				raise abort(f"No such repository {repo_name} for {'org' if org else 'user'} {user.login}.")
 
 			self.update_topics(repo)
 			click.echo(f"Success! View the repository online at {repo.html_url}")
@@ -209,29 +192,35 @@ class GitHubManager(RepoHelper):
 		:param org: Whether the repository should be created for the organisation set as ``username``,
 			or for the authenticated user (default).
 
-		.. versionchanged:: 0.4.1
+		.. versionchanged:: 0.5.0
 
 			Removed the ``verbose`` option. Provide it to the class constructor instead.
 
-		.. versionchanged:: 0.4.1  Added the ``org`` argument.
+		.. versionchanged:: 0.5.0  Added the ``org`` argument.
 		"""
 
 		with self.echo_rate_limit():
-
 			user = self.get_org_or_user(org)
+			repo_name = self.templates.globals["repo_name"]
+			repo: Optional[repos.Repository] = self.github.repository(user.login, repo_name)
 
-			try:
-				repo = user.get_repo(self.templates.globals["repo_name"])
-			except GithubException as e:
-				self.handle_exception(e)
+			if repo is None:
+				raise abort(f"No such repository {repo_name} for {'org' if org else 'user'} {user.login}.")
+
+			# TODO: add config option for allow_merge_commit
+
+			repo.edit(
+					name=repo.name,
+					**self.get_repo_kwargs(),
+					allow_merge_commit=False,
+					)
 
-			repo.edit(**self.get_repo_kwargs())
 			self.update_topics(repo)
 			click.echo("Up to date!")
 
 		return 0
 
-	def get_org_or_user(self, org: bool = False) -> Union[Organization, AuthenticatedUser]:
+	def get_org_or_user(self, org: bool = False) -> Union[orgs.Organization, users.User]:
 		"""
 		If ``org`` is :py:obj:`True`, returns the :class:`~.Organization` object representing the
 		GitHub org that owns the repository.
@@ -241,14 +230,14 @@ class GitHubManager(RepoHelper):
 
 		:param org:
 
-		.. versionadded:: 0.4.1
+		.. versionadded:: 0.5.0
 		"""  # noqa: D400
 
-		user = self.github.get_user()
+		user = get_user(self.github)
 
 		if org:
 			self.assert_org_member(user)
-			return self.github.get_organization(self.templates.globals["username"])
+			return self.github.organization(self.templates.globals["username"])
 		else:
 			self.assert_matching_usernames(user)
 			return user
@@ -270,31 +259,24 @@ class GitHubManager(RepoHelper):
 
 		``PYPI_TOKEN`` and ``ANACONDA_TOKEN`` can either be passed as keyword arguments to this function or provided at the interactive prompt.
 
-		.. versionadded:: 0.4.1
-
-		.. versionchanged:: 0.4.1  Add ``overwrite``, ``PYPI_TOKEN``, ``ANACONDA_TOKEN`` options.
+		.. versionadded:: 0.5.0
+		.. versionchanged:: 0.5.0  Add ``overwrite``, ``PYPI_TOKEN``, ``ANACONDA_TOKEN`` options.
 		"""
 
 		with self.echo_rate_limit():
 			user = self.get_org_or_user(org)
+			repo_name = self.templates.globals["repo_name"]
 
-			try:
-				repo: Repository = user.get_repo(self.templates.globals["repo_name"])
-			except GithubException as e:
-				self.handle_exception(e)
+			repo: Optional[repos.Repository] = self.github.repository(user.login, repo_name)
 
-			repos_url = RequestsURL("https://api.github.com/repos")
-			secrets_url = repos_url / repo.owner.login / repo.name / "actions/secrets"
-			secrets_url.session.headers = {
-					"Authorization": repo._requester._Requester__authorizationHeader,  # type: ignore
-					"user-agent": "repo_helper_github",
-					}
+			if repo is None:
+				raise abort(f"No such repository {repo_name} for {'org' if org else 'user'} {user.login}.")
 
 			# List of existing secrets.
-			existing_secrets = [secret["name"] for secret in secrets_url.get().json()["secrets"]]
+			existing_secrets = secrets.get_secrets(repo)
 
 			# Public key to encrypt secrets with.
-			public_key = (secrets_url / "public-key").get().json()
+			public_key = secrets.get_public_key(repo)
 
 			ret = 0
 			target_secrets = {"PYPI_TOKEN"}
@@ -314,15 +296,13 @@ class GitHubManager(RepoHelper):
 				if update:
 					operation = "update" if secret_name in existing_secrets else "create"
 
-					encrypted_value = encrypt_secret(
-							public_key["key"],
-							secret_value=locals().get(secret_name, None) or getpass(f"{secret_name}: "),
+					response = secrets.set_secret(
+							repo,
+							secret_name=secret_name,
+							value=locals().get(secret_name, None) or getpass(f"{secret_name}: "),
+							public_key=public_key,
 							)
 
-					key_id = public_key["key_id"]
-					secret_json = {"encrypted_value": encrypted_value, "key_id": key_id}
-					response = (secrets_url / secret_name).put(json=secret_json)
-
 					if response.status_code not in {200, 201, 204}:
 						message = f"Could not {operation} the secret {secret_name!r}: Status {response.status_code}"
 						click.echo(Fore.YELLOW(message), color=self.colour)
@@ -343,34 +323,27 @@ class GitHubManager(RepoHelper):
 		:param org: Whether the repository should be created for the organisation set as ``username``,
 			or for the authenticated user (default).
 
-		.. versionadded:: 0.4.1
+		.. versionadded:: 0.5.0
 		"""
 
 		with self.echo_rate_limit():
 			user = self.get_org_or_user(org)
+			repo_name = self.templates.globals["repo_name"]
 
-			try:
-				repo: Repository = user.get_repo(self.templates.globals["repo_name"])
-			except GithubException as e:
-				self.handle_exception(e)
+			repo: Optional[repos.Repository] = self.github.repository(user.login, repo_name)
 
-			gh_branch = repo.get_branch(branch)
+			if repo is None:
+				raise abort(f"No such repository {repo_name} for {'org' if org else 'user'} {user.login}.")
+
+			gh_branch: Optional[Branch] = repo.branch(branch)
 			required_checks = list(compile_required_checks(self))
 
-			# gh_branch.edit_required_status_checks(strict=False, contexts=)
-			gh_branch.edit_protection(
-					strict=False,
-					contexts=required_checks,
-					dismiss_stale_reviews=False,
-					required_approving_review_count=1,
-					)
-
-			gh_branch.edit_protection(strict=False, contexts=required_checks)
+			protect_branch(gh_branch, status_checks=required_checks)
 
 		click.echo("Up to date!")
 		return 0
 
-	def assert_matching_usernames(self, user: AuthenticatedUser):
+	def assert_matching_usernames(self, user: users.User):
 		"""
 		Assert that the username configured in ``repo_helper.yml`` matches that of the authenticated user.
 
@@ -384,22 +357,28 @@ class GitHubManager(RepoHelper):
 					f"If {self.templates.globals['username']} is an organisation you should use the --org flag."
 					)
 
-	def assert_org_member(self, user: AuthenticatedUser):
+	def assert_org_member(self, user: users.User):
 		"""
 		Assert that the organisation configured in ``repo_helper.yml`` exists, and the authenticated user is a member.
 
 		:param user:
 		"""
 
-		try:
-			user.get_organization_membership(self.templates.globals["username"])
-		except UnknownObjectException:
+		def error():
 			raise abort(
-					f"Either organisation configured in 'repo_helper.yml' ({self.templates.globals['username']}) "
+					f"Either the organisation configured in 'repo_helper.yml' ({self.templates.globals['username']}) "
 					f"does not exist or the authenticated user ({user.login}) is not a member!"
 					)
 
-	def update_topics(self, repo: Repository):
+		try:
+			org = self.github.organization(self.templates.globals["username"])
+		except NotFoundError:
+			raise error()
+
+		if not org.is_member(user.login):
+			raise error()
+
+	def update_topics(self, repo: repos.Repository):
 		"""
 		Update the repository's topics.
 
@@ -407,62 +386,68 @@ class GitHubManager(RepoHelper):
 		"""
 		# TODO: other languages detected in repo
 
-		topics = set(repo.get_topics())
+		raw_topics = repo.topics()
+		if raw_topics is None:
+			topics = set()
+		else:
+			topics = set(raw_topics.names)
+
 		topics.add("python")
 		topics.update(self.templates.globals["keywords"])
 		repo.replace_topics(sorted(map(_lower, topics)))
 
-	@staticmethod
-	def handle_exception(exc: GithubException) -> NoReturn:
-		"""
-		Handle an exception raised by the GitHub REST API.
-
-		:param exc:
-
-		:raises: :class:`click.Abort`
-		"""
-
-		data: _ExcData = exc.data  # type: ignore
-
-		if "errors" in data:
-			errors = DelimitedList(i["message"] for i in data["errors"])
-		else:
-			errors = DelimitedList([data["message"]])
-
-		raise abort(f"{exc.data['message']}\n{errors:\t\n}")
-
 	def get_repo_kwargs(self) -> _EditKwargs:
 		"""
 		Returns the keyword arguments used when creating and updating repositories.
 		"""
 
-		# TODO: add config option for allow_merge_commit
-
-		edit_kwargs: _EditKwargs = {
-				"description": self.templates.globals["short_desc"],
-				"allow_merge_commit": False,
-				}
+		edit_kwargs: _EditKwargs = {"description": self.templates.globals["short_desc"]}
 
 		if self.templates.globals["enable_docs"]:
 			edit_kwargs["homepage"] = self.templates.globals["docs_url"]
 
 		return edit_kwargs
 
+	def create_labels(self, org: bool = False) -> int:
+		"""
+		Create labels for this repository.
 
-def encrypt_secret(public_key: str, secret_value: str) -> str:
-	"""
-	Encrypt a GitHub Actions secret.
+		:param org: Whether the repository should be created for the organisation set as ``username``,
+			or for the authenticated user (default).
 
-	:param public_key:
-	:param secret_value:
+		.. versionadded:: 0.5.0
+		"""
 
-	.. versionadded: 0.4.1
-	"""
+		with self.echo_rate_limit():
+			user = self.get_org_or_user(org)
+			repo_name = self.templates.globals["repo_name"]
+			repo: Optional[repos.Repository] = self.github.repository(user.login, repo_name)
 
-	public_key = public.PublicKey(public_key.encode("utf-8"), encoding.Base64Encoder())
-	sealed_box = public.SealedBox(public_key)
-	encrypted = sealed_box.encrypt(secret_value.encode("utf-8"))
-	return b64encode(encrypted).decode("utf-8")
+			if repo is None:
+				raise abort(f"No such repository {repo_name} for {'org' if org else 'user'} {user.login}.")
+
+			current_labels = {label.name: label for label in repo.labels()}
+
+			for label in check_status_labels.values():
+				if label.name in current_labels:
+					current_labels[label.name].update(**label.to_dict())
+				else:
+					label.create(repo)
+					click.echo(f"Created label {label.name}")
+
+		click.echo("Up to date!")
+
+		return 0
+
+
+encrypt_secret = deprecated(
+		deprecated_in="0.5.0",
+		removed_in="1.0.0",
+		current_version=__version__,
+		details="Use the new 'github3-utils' package instead."
+		)(
+				secrets.encrypt_secret
+				)
 
 
 def compile_required_checks(repo: RepoHelper) -> Iterator[str]:
@@ -471,7 +456,7 @@ def compile_required_checks(repo: RepoHelper) -> Iterator[str]:
 
 	:param repo:
 
-	.. versionadded:: 0.4.1
+	.. versionadded:: 0.5.0
 	"""
 
 	actions_manager = ActionsManager(repo.target_repo, repo.templates)
@@ -498,7 +483,7 @@ def compile_required_checks(repo: RepoHelper) -> Iterator[str]:
 
 			yield f"{ci_platform} / Python {version}"
 
-	yield from ["mypy / ubuntu-latest", "Flake8"]
+	yield from [f"mypy / {platform_ci_names['Linux']}", "Flake8"]
 
 	if repo.templates.globals["enable_docs"]:
 		yield "docs"
@@ -520,7 +505,7 @@ class IsolatedGitHubManager(GitHubManager):
 	:param verbose: Whether to show information on the GitHub API rate limit.
 	:param colour: Whether to use coloured output.
 
-	.. versionadded:: 0.4.1
+	.. versionadded:: 0.5.0
 	"""  # noqa: D400
 
 	def __init__(
@@ -531,21 +516,21 @@ class IsolatedGitHubManager(GitHubManager):
 			*,
 			managed_message="This file is managed by 'repo_helper'. Don't edit it directly.",
 			verbose: bool = False,
-			colour: Optional[bool] = True,
+			colour: ColourTrilean = True,
 			):
 
 		self._tmpdir = tempfile.TemporaryDirectory()
 
-		self.github = Github(token)
+		self.github = Github(token=token)
 		self.verbose = verbose
 		self.colour = resolve_color_default(colour)
 
 		target_repo = PathPlus(self._tmpdir.name)
 		config_file_name = "repo_helper.yml"
 
-		github_repo = self.github.get_repo(f"{username}/{repo_name}")
-		contents_from_github: ContentFile = github_repo.get_contents(config_file_name)  # type: ignore
-		(target_repo / config_file_name).write_bytes(contents_from_github.decoded_content)
+		github_repo: repos.Repository = self.github.repository(username, repo_name)
+		contents_from_github: contents.Contents = github_repo.file_contents(config_file_name)
+		(target_repo / config_file_name).write_bytes(contents_from_github.decoded)
 
 		RepoHelper.__init__(self, target_repo, managed_message)
 
